# biasfix.cl-- fix the 1200x800's bias discontinuities.
#
# This script deals with the problem of the ~3 ADU discontinuities in the bias 
# level shown by the 1200x800 CCD when binning.  The location of the 
# discontinuity is roughly dependent on the exposure time but will wander 
# slightly and occasionally will jump by over 100 pixels.  Fortunately the 
# problem seems to be removable (to first order) using the overscan region.  
#
# 1) Examine each image to identify the TOP row of the BOTTOM bias section,
#    if there is a jump in the bias level at all.
# 2) Copy the images to a safe location, since the biasfix (and ccdproc)
#    operations done "in place".
# 3) Run this routine, which computes the mean of 2 blocks of pixels, one each 
#    just below and just above the discontinuity, calculates the difference,
#    and adds that # constant into the entire bottom half of the image.  
# 4) Do the first pass through "ccdproc".  The overscan correction in this
#    pass (polynomial-fitting) can proceed normally.
#
# 21/04/93 created by Brian McLeod

procedure biasfix(imname,toprow)
string imname {prompt="Name of image"}
int toprow {prompt="Top row of bottom section"}
int numrows {20,prompt="Number of rows to use for computing statistics"}
int leftcol {603,prompt="Leftmost column to use for statistics"}
int rightcol {620,prompt="Rightmost column to use for statistics"}

begin

string iname,tname
int ttoprow
real bot,top,diff
string botsection,topsection,botfixsection

iname = imname
ttoprow=toprow

tname = mktemp("12x8")

botsection = "["//leftcol//":"//rightcol//","//ttoprow-numrows+1//":"//ttoprow//"]"
topsection = "["//leftcol//":"//rightcol//","//ttoprow+1//":"//ttoprow+numrows//"]"
botfixsection = "[*,1:"//ttoprow//"]"

print(botsection)
print(topsection)
print(botfixsection)

imstat (iname//botsection,
fields="mean", lower=INDEF, upper=INDEF, binwidth=0.1, format=no) | scan(bot)

imstat (iname//topsection,
fields="mean", lower=INDEF, upper=INDEF, binwidth=0.1, format=no) | scan(top)

diff = bot - top

print (bot,top,diff)
imarith (iname//botfixsection,
"-", diff, tname, title="", divzero=0., hparams="", pixtype="", calctype="",
verbose=no, noact=no)

imcopy (tname,iname//botfixsection, verbose=no)

imdelete(tname,ver-)

end

