# bindata.cl -- bin data and plot histograms of it
# 
# This program reads in one or two columns of data (call them 1 and 2),
# bins them as specified, and outputs one of three histogram types:
# 	normal plots 1 (cumulative: sum1)
# 	percent plots 1/2 (cumulative: (sum1)/(sum2)
# 	sumpercent plots 1/(1+2) (cumulative: (sum1)/(sum(1+2)))
#
# Currently there is a limit of 1000 bins.
#
# 950724: created by Pat Hall
# 950725: autoscaling ability & other minor bits added 
# 960226: option to print midpoint of bin as x coordinate
# 970104: numerous revisions to (auto)scaling

procedure bindata (histype, cumulative, autoscale, binwidth, firstfile, firstcol)

string   histype {enum="normal|percent|sumpercent",prompt="Histogram type"}
bool  cumulative {prompt="Cumulative histogram?"}
bool   autoscale {prompt="Automatically determine start & end bin values?"}
real    binwidth {prompt="Bin width"}
string firstfile {prompt="File containing first variable"}
int   firstcol=1 {prompt="Column containing first variable"}
string secondfile {prompt="File containing second variable",mode="q"}
int   secondcol=1 {prompt="Column containing second variable",mode="q"}
real 	z1	  {prompt="Starting value for bins",mode="q"}
real  	z2 	  {prompt="Ending value for bins",mode="q"}
bool   midpt=yes  {prompt="List midpoint coord instead of start for each bin?"}
string outfile="" {prompt="List out histogram to this filename"}
bool   include=no {prompt="Include z2 in top bin?"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct  *name

begin

	real	low, high, wide, val, x, plotval, xval, graphval, a, b, c
	real    numer, denom, numerold, denomold
	real	bin1[1000], bin2[1000]
	int	onecol, twocol, z, y
	string  hertype, test0, test1, test2, file1, file2, ooutfile
	bool	go, ccumul, aauto, iinclude, mmidpt
#	struct	val

	hertype = histype
	ccumul = cumulative
	aauto = autoscale
	if(!aauto){
		low = z1
		high = z2
	}
	wide = binwidth
	file1 = firstfile
	onecol = firstcol
	if(hertype!="normal"){
		file2 = secondfile
		twocol = secondcol
	}
        mmidpt=midpt
	ooutfile=outfile
	if(ooutfile=="")ooutfile="_temp_file_"
	del("_temp_file_", ver-, >& "dev$null")
	iinclude=include

# make sure necessary packages are loaded	
	go = yes
#	if(!defpac("focas")) {go=no}
	if(go) {

#setup 
if(aauto){
	test0=mktemp("auto")
	if(hertype=="normal"){
#		copy(file1, test0, ver-, >& "dev$null")
		fields(file1, onecol, lines=1-999999999, >> test0)
	} else {
		concat(file1//","//file2, "._test0")
		fields("._test0", onecol, lines=1-999999999, >> test0)
	}
#	sort (test0, col=onecol, num+, rev-, >> test0//"x")
	sort (test0, col=1, num+, rev-, >> test0//"x")
	head (test0//"x", nlines=1) | scan(low)
	tail (test0//"x", nlines=1) | scan(high)
	z=1+int((high-low)/wide) 			#number of bins
} else {
	z=int((high-low)/wide) 			#number of bins
}
#print(z)
bin1=0.
bin2=0.
x=0
y=0

#read first dataset
test1=mktemp(file1)
fields(file1, onecol, >> test1)
name=test1
while (fscan (name, val) !=EOF) {
	x=1+int((val - low)/wide)  		#= 1,2,... (bin number)
	if(iinclude){
		if(val==z2)x=z
	}
#	print(x,bin1[x])
	bin1[x]=bin1[x]+1
#	print(bin1[x])
	}

#read second dataset, if necessary
if(hertype!="normal"){
	test2=mktemp(file2)
	fields(file2, twocol, >> test2)
	name = test2
	while (fscan (name, val) !=EOF) {
		x=1+int((val - low)/wide)  		#= 1,2,... (bin number)
		bin2[x]=bin2[x]+1
	}
}

#Compute and output plotting value for each bin
plotval=0			#start
numerold=0			#start
denomold=0			#start
for (y=1; y<=z; y+=1) {
graphval=0.
if (mmidpt) {
#middle
	xval=((y-1)*wide)+low+(wide/2.)
} else {
#start
	xval=((y-1)*wide)+low
}
#print(xval, bin1[y], bin2[y])
if(hertype=="normal")graphval=bin1[y]
#Avoid division by zero by setting 0/0=0 or x/0=1
if(hertype=="percent"){
	if(bin2[y]==0.){
		if(bin1[y]!=0.){
			graphval=1.
			}
		else{
			graphval=0.
			}
		}
	else{
		graphval=bin1[y]/bin2[y]
	}
}
#Avoid division by zero by setting 0/0=0 or x/0=1
if(hertype=="sumpercent"){
	if((bin1[y]+bin2[y])==0.){
		if(bin1[y]!=0.){
			graphval=1.
			}
		else{
			graphval=0.
			}
	}
	else{
		graphval=bin1[y]/(bin1[y]+bin2[y])
	}
}
#Sum appropriately if cumulative=yes
if(ccumul){
	numer=bin1[y]
	if(hertype=="percent"){
		denom=bin2[y]
	} else {
		denom=bin1[y]+bin2[y]
	}
	if(hertype=="normal"){
		plotval=plotval+numer
	} else {
		numerold=numerold+numer
		denomold=denomold+denom
		if(denomold==0){
			if(numerold==0){
				plotval=plotval+0
			} else {
				plotval=1
			}
		} else {
			plotval=numerold/denomold
		}
	}
} else {
	plotval=graphval
}

print(xval,plotval, >> ooutfile)
#next y
}

#Plot the histogram and delete plotting file if list=no
graph(ooutfile,point-)
if(ooutfile=="_temp_file_")delete(ooutfile, ver-, >& "dev$null")

}

else {

	#print("WARNING: package FOCAS is not loaded, or package SYSTEM is.")
        #print("Please load FOCAS or unload SYSTEM and restart this task.")
	}

if(aauto)del(test0//"*", ver-, >& "dev$null")
del(test1, ver-, >& "dev$null")
if(hertype!="normal")del(test2, ver-, >& "dev$null")
del("_temp_file_", ver-, >& "dev$null")
if(echog)beep

end

