procedure focasflat (rootname, routname)

# program to flatten a big (~2048**2) CCD image, ~20 minutes per image.

# requires iterstat, iraf2unix and mysetcat

string  rootname	{prompt="Name of original image (no .imh included)"}
string  routname	{prompt="Name of output image (no .imh included)"}
#bool check=yes		{prompt="Inspect sky image before continuing?"}
#bool doit=yes		{prompt="Do subtraction?"}
bool   echog        {yes,prompt="Beep when task is done?"}

# int_time 	is the integration time in the filter
# o 		is the total number of offset images in the filter
# j 		is the number of the offset image whose parameters are specified in the particular line of the file
# sky_sigma_j 	is the sky sigma of the image
# f 		is the filter code letter, running in the order ubvriI

begin

	real	ftime, sig, a, mn
	int	xmax, ymax, k, word, chars, num1, num2, st, d, o
	string	image, root, l, catpars, tempname, i, h, f, m, rroutname
	struct	c
	file    temp.file, tmpsfiles 

	root = rootname
	rroutname = routname
		imgets(root, "i_naxis1")
		xmax = real(imgets.value)
		imgets(root, "i_naxis2")
		ymax = real(imgets.value)
		imgets(root, "exptime")
		ftime = real(imgets.value)
		imgets(root, "itersig")
		if (imgets.value=="0") {
			iterstat (root, addh+, ver-, print-)
			imgets(root, "itersig")
			sig = real(imgets.value)
			imgets(root, "itermean")
			mn = real(imgets.value)
		} else {
			sig = real(imgets.value)
			imgets(root, "itermean")
			mn = real(imgets.value)
		}

	    print ("Requires iraf2unix and mysetcat be in this directory...")
	    print ("Now working on image "//root//".  Rotating images...")
	    imdel ("rtff*.imh", ver-, >& "dev$null")
 	    imcopy (root, "rtff"//"a", verbose-); imtrans ("rtff"//"a[*,-*]", "rtff"//"b")
 	    imtrans ("rtff"//"b[*,-*]", "rtff"//"c"); imtrans ("rtff"//"c[*,-*]", "rtff"//"d")

# flatten the images using the FOCAS-based flattening routine.
 	    for (k=1; k <= 4; k+=1) {
	 	if (k<1.4)
		 	m = "a"
	 	if (k>1.5)
		 	m = "b"
	 	if (k>2.5)
		 	m = "c"	
	 	if (k>3.5)
			m = "d"
		del ("temp.file", ver-, >& "dev$null")

print ("Flattening image "//m//" using the FOCAS-based flattening routine...")
		print ("rtff"//m//".cat", " ", "rtff"//m, " ", "f", " ", ftime, " ", sig, >> "temp.file")
		!set catpars=`awk -f iraf2unix temp.file`; mysetcat $catpars
#		detect ("rtff"//m//".cat", "1", ymax, mn-5*sig, mn+5*sig)
		print ("detect rtff"//m//".cat 1 "//ymax//" "//mn-5*sig//" "//mn+5*sig, >> "do_detmp.cl")
		cl (< "do_detmp.cl")
		del ("do_detmp.cl", ver-)
		sky ("rtff"//m//".cat")
		evaluate ("rtff"//"a.cat")
		del ("temp.file", ver-, >& "dev$null")
	    }
	    imdel ("rtff"//"b.imh", ver-); imdel ("rtff"//"c.imh", ver-); imdel ("rtff"//"d.imh", ver-)

# rotate sky images back to common orientation
print ("Rotating sky images back to common orientation...")
	    imtrans ("rtff"//"d.sky[*,-*]", "rtff"//"d.sky"); imtrans ("rtff"//"b.sky[-*,*]", "rtff"//"b.sky")
	    imtrans ("rtff"//"c.sky[-*,*]", "rtff"//"c.sky"); imtrans ("rtff"//"c.sky[-*,*]", "rtff"//"c.sky")
	    files ("rtff"//"*.sky.imh", > "tmpsfiles")
# combine them into one
print ("Combining sky images into one...")
	    imcom ("@tmpsfiles", "rtff"//".avsky", combine="median", reject="minmax", nlow=0, nhigh=1, scale="none", weight="none", zero="none")
# scale it to the original image size
print ("Scaling sky image to original image size...")
	    blkrep ("rtff"//".avsky", "rtff"//".avskye", 8, 8)
	    imstat ("rtff"//".avsky", fields="mode", format-) | scan (a)
	    imdel ("rtff"//".avsky", ver-)
	imcopy("rtff"//".avskye[1:"//xmax//",1:"//ymax//"]", root//".sky", ver-)
	    imdel("rtff"//".avskye", ver-)
#--check/doit here
# clean up
	    	imdel ("@tmpsfiles", ver-); imdel ("rtff"//"*.sky.imh", ver-)
del ("tmpsfiles", ver-); del ("rtff"//"*.ar", ver-); del ("rtff"//"*.cat", ver-)
# subtract the focas sky image from the original image & add back in the mode
print ("Subtracting sky image from original & adding mode back in...")
	    	imar ("rtff"//"a", "-", root//".sky", rroutname)
	    	imdel ("rtff"//"a.imh", ver-); imdel ("rtff"//"*sky.imh", ver-)
	    	imar (rroutname, "+", a, rroutname)
	    	minmax (rroutname, force+)
		iterstat (rroutname, addh+, ver-, print-)
	    
if(echog)beep

end
