procedure focasflat (rootname, routname)

# program to flatten a big (~2048^2) CCD image, ~20 minutes per image.

# requires focas, iterstat, iraf2unix and mysetcat
# 981108: added check for above, and copying if needed.
# 981109: added stop if FOCASFLT header parameter already exists
# 981110: added optional check and doit step, and if the ALIGNSEC header
#  contains "->" (which indicates the data does not occupy the full image,
#  i.e. the I-band doesn't cover the full B & R-band overlap region), then
#  iterstat is done on the good data region only (taken from ALIGNSEC)
# 990808: now deletes "tmpsfiles" before attempting to create it
# 000828: explicit display of final iterated statistics

string  rootname	{prompt="Name of original image (no .imh included)"}
string  routname	{prompt="Name of output image (no .imh included)"}
bool    check		{no,prompt="Inspect sky image before continuing?"}
bool    doit		{yes,prompt="Do subtraction after inspection?",mode="q"}
bool    echog        	{yes,prompt="Beep when task is done?"}

begin

	real	ftime, sig, a, mn
	int	xmax, ymax, k, word, chars, num1, num2, st, d, o
	string	image, root, l, catpars, tempname, i, h, f, m, rroutname
	string	alignsec
	bool	go, ddoit, cheque
	struct	c
	file    temp.file, tmpsfiles 

	root = rootname
	rroutname = routname//".imh"
		imgets(root, "i_naxis1")
		xmax = real(imgets.value)
		imgets(root, "i_naxis2")
		ymax = real(imgets.value)
		imgets(root, "exptime")
		ftime = real(imgets.value)
		imgets(root, "itersig")
		if (imgets.value!="0") {
			sig = real(imgets.value)
			imgets(root, "itermean")
			mn = real(imgets.value)
		} else {
			imgets(root, "alignsec")
			if ((stridx("->",imgets.value))!=0) {
  alignsec=substr(imgets.value,3+stridx("->",imgets.value),strlen(imgets.value))
				iterstat(root//alignsec,addh+,ver-,print-)
			} else {
				iterstat (root, addh+, ver-, print-)
			}
			imgets(root, "itersig")
			sig = real(imgets.value)
			imgets(root, "itermean")
			mn = real(imgets.value)
		}
	cheque=check
	ddoit=yes
	
	go=yes
	if(!defpac("focas")) {go=no; print "FOCAS package not loaded!"}
	imgets (root, "FOCASFLT", >& "dev$null")
        if((imgets.value)!="0") {go=no; print "FOCASFLAT already run on image!"}
	if(go) {
	    if(!access("iraf2unix")) {copy ("phat$src/iraf2unix", ".", ver+)}
	    if(!access("mysetcat")) {copy ("phat$src/mysetcat", ".", ver+)}

	    print ("Now working on image "//root//".  Rotating images...")
	    imdel ("rtff*.imh", ver-, >& "dev$null")
 	    imcopy (root, "rtff"//"a.imh", verbose-); imtrans ("rtff"//"a.imh[*,-*]", "rtff"//"b.imh")
 	    imtrans ("rtff"//"b.imh[*,-*]", "rtff"//"c.imh"); imtrans ("rtff"//"c.imh[*,-*]", "rtff"//"d.imh")

# flatten the images using the FOCAS-based flattening routine.
 	    for (k=1; k <= 4; k+=1) {
	 	if (k<1.4)
		 	m = "a"
	 	if (k>1.5)
		 	m = "b"
	 	if (k>2.5)
		 	m = "c"	
	 	if (k>3.5)
			m = "d"
		del ("temp.file", ver-, >& "dev$null")

print ("Running FOCAS on image "//m//" to create its sky image...")
		print ("rtff"//m//".cat", " ", "rtff"//m, " ", "f", " ", ftime, " ", sig, >> "temp.file")
		!set catpars=`awk -f iraf2unix temp.file`; mysetcat $catpars
		print ("detect rtff"//m//".cat 1 "//ymax//" "//mn-5*sig//" "//mn+5*sig, >> "do_detmp.cl")
		cl (< "do_detmp.cl")
		del ("do_detmp.cl", ver-)
		sky ("rtff"//m//".cat")
		evaluate ("rtff"//"a.cat")
		del ("temp.file", ver-, >& "dev$null")
	    }
	    imdel ("rtff"//"b.imh", ver-); imdel ("rtff"//"c.imh", ver-); imdel ("rtff"//"d.imh", ver-)

# rotate sky images back to common orientation
print ("Rotating sky images back to common orientation...")
	    imtrans ("rtff"//"d.sky.imh[*,-*]", "rtff"//"d.sky.imh"); imtrans ("rtff"//"b.sky.imh[-*,*]", "rtff"//"b.sky.imh")
	    imtrans ("rtff"//"c.sky.imh[-*,*]", "rtff"//"c.sky.imh"); imtrans ("rtff"//"c.sky.imh[-*,*]", "rtff"//"c.sky.imh")
	    del ("tmpsfiles", ver-); files ("rtff"//"*.sky.imh", > "tmpsfiles")
# combine them into one
print ("Combining sky images into one...")
	    imcom ("@tmpsfiles", "rtff"//".avsky.imh", combine="median", reject="minmax", nlow=0, nhigh=1, scale="none", weight="none", zero="none")
# scale it to the original image size
print ("Scaling sky image to original image size...")
	    blkrep ("rtff"//".avsky.imh", "rtff"//".avskye.imh", 8, 8)
	    imstat ("rtff"//".avsky.imh", fields="mode", format-) | scan (a)
	    imdel ("rtff"//".avsky.imh", ver-)
imcopy("rtff"//".avskye.imh[1:"//xmax//",1:"//ymax//"]", root//".sky.imh", ver-)
	    imdel("rtff"//".avskye.imh", ver-)

# check sky image before continuing
	    if(cheque) {
		print ("Displaying original image and focasflat sky image...")
		display (root, 1)
		display (root//".sky.imh", 2)
	    	ddoit=doit
	    }

# clean up temporary images
	    imdel ("@tmpsfiles", ver-); imdel ("rtff"//"*.sky.imh", ver-)
del ("tmpsfiles", ver-); del ("rtff"//"*.ar", ver-); del ("rtff"//"*.cat", ver-)
# subtract the focas sky image from the original image & add back in the mode
	    if(ddoit){
	  print ("Subtracting sky image from original & adding mode back in...")
	    	imar ("rtff"//"a", "-", root//".sky", rroutname)
	    	imdel ("rtff"//"a.imh", ver-); imdel ("rtff"//"*sky.imh", ver-)
	    	imar (rroutname, "+", a, rroutname)
	    	minmax (rroutname, force+)
		imgets(rroutname, "alignsec")
                if ((stridx("->",imgets.value))!=0) {
  alignsec=substr(imgets.value,3+stridx("->",imgets.value),strlen(imgets.value))
                	iterstat (rroutname//alignsec,addh+,ver-,print-)
                } else {
                	iterstat (rroutname, addh+, ver-, print-)
                }
		hedit (rroutname,"FOCASFLT","yes",add+,del-,ver-,show+,update+)
	    } else {
		print ("Sky image not subtracted.")
	    	imdel ("rtff"//"a.imh", ver-); imdel ("rtff"//"*sky.imh", ver-)
	    } 

if(echog)beep

} else {
	beep
}

end


