#{ AVERAGE -- Compute the average and standard deviation of a list of numbers.
# Numeric input is via the standard input (one number per line).  The mean,
# sigma, and number of samples are written to the standard output.
#
# By default (Average called with no arguments), the sample is taken
# to be the set of numbers in the standard input when Average is run.  
# Additional points can be added to or deleted from the sample by
# rerunning Average with one of the following arguments:
# 
#	add - add points to the sample, recalculate mean and sigma
#	sub - subtract points from the sample


{
	if ($nargs == 0)		# if no arguments, don't query
	    opstring = "new_sample"
	else
	    opstring = option		# get parameter "option"


	if (opstring == "add")		# add, subtract, or start over?
	    addsub_flag = 1
	else if (opstring == "sub" || opstring == "subtract")
	    addsub_flag = -1
	else {				# new sample (default)
	    addsub_flag = 1
	    n = 0			# number of points in sample
	    sum = 0
	    sumsqrs = 0
	    variance = 0
	    sigma = 0
	    mean = 0
	}

# Process data and save to temporary file
del ("__iteravgtmp", ver-, >& "dev$null")
	while (scan (data_value) != EOF)
	    if (nscan() == 1 && data_value != INDEF) {
		n += addsub_flag
		sum += addsub_flag * data_value 
		sumsqrs += addsub_flag * data_value * data_value
		print (data_value, >> "__iteravgtmp")
	    }

# Iterate
    iter=0
    nold=0
    minval=-100000000000000000000
    maxval=100000000000000000000
    for (iter=0; iter<=maxiter; iter+=1) {

        list="__iteravgtmp"
	if(iter>0) {
	    while (fscan (list, data_value) != EOF) {
	if (data_value != INDEF && data_value<=maxval && data_value>=minval) {
			n += addsub_flag
			sum += addsub_flag * data_value 
			sumsqrs += addsub_flag * data_value * data_value
		}
	    }
	}

# Find mean, var, sig
	if (n > 1) {
	    mean = sum / n
	    variance = (n * sumsqrs - sum * sum) / (n * (n-1))
	    if (variance < 0)		# possible with roundoff error
		sigma = 0.0
	    else
		sigma = sqrt (variance)
	}
   
# End loop?
	if ((n != 0 && n == nold) || iter==maxiter) {
		iter=maxiter+1
	} else {
		nold=n
		maxval=mean+nsigrej*sigma
		minval=mean-nsigrej*sigma
		n=0
		sum=0
		sumsqrs=0
		mean=0
		sigma=0
		variance=0
        }
    }
#type ("__iteravgtmp", >> "oog")
#count ("__iteravgtmp")
del ("__iteravgtmp", ver-, >& "dev$null")

# print output
	if (n <= 0)
	    print ("INDEF ", "INDEF ", n)
	else if (n == 1)
	    print (sum, "INDEF ", 1)
	else {
	    mean = sum / n
	    variance = (n * sumsqrs - sum * sum) / (n * (n-1))
	    if (variance < 0)		# possible with roundoff error
		sigma = 0.0
	    else
		sigma = sqrt (variance)
	    print (mean, sigma, n)
	}

}
