# mkfringe.cl -- create fringe frame & optionally subtract it from images
#
# Calls routines iterstat, addcomment, noao.imred.ccdred.mkskycor,
# and possibly stsdas.toolbox.imgtools.imcalc
#
# CHANGES:
# 98/09/14: created from defringe.cl

procedure mkfringe(rootname,imagelist,logfile,defstring,biasimage)

string rootname   {prompt="Rootname for output images and image list"}
string imagelist  {prompt="List of images to process (do not use an `@')"}
string logfile    {prompt="Name of log file"}
string defstring  {prompt="String to be appended to output images"}
string biasimage  {prompt="Zero level calibration image for ccdproc"}
bool   getfringe  {yes,prompt="Do fringe creation step?"}
bool   imdisplay  {yes,prompt="Display sky+fringe image?"}
bool   dofringe   {yes,prompt="Do fringe removal step?"}
real   gain       {prompt="Gain of CCD (for ccdclip in imcombine)"}
real   rdnoise    {prompt="Read noise of CCD (for ccdclip in imcombine)"}
bool   forcescale {no,prompt="Force recalculation of image scales?"}
bool goon=yes {prompt="Continue with task using this sky+fringe image?",mode="q"}
bool   ssave=no   {prompt="Save sky+fringe image?",mode="q"}
bool   echog        {yes,prompt="Beep when task is done?"}
struct *list
struct *list2
struct *list3
struct *datelist

begin

string ddefstring, ffringename, outlist, sskyfringename, sskyname, biasimg
string rrootname, ddark, llogfile, bbadmask, eextension, normfile, img
string flatlist, flat, flatbase, meanfile, currentflat, base, mmaskname
string iimagelist, FlatImageList, fflatstr, check, ttplname, templist, flatdiv
string datefile, thedate, ickinfo, name, tempstring, otherstring, tempfile
real mean, nnumber, flatnorm, valmed, fringmed, ggain, rrdnoise, scale, scale2
real indiv_exptime, avg_exptime, sum_exptime
int sstartno, eendno, i, j, numpix, x, y, z
bool zzeroout, ttpl, sstdpars, go, fflatsave, force, ggoon

rrootname = rootname
iimagelist = imagelist
llogfile = logfile
ddefstring = defstring
biasimg = biasimage
ggain = gain
rrdnoise = rdnoise
force = forcescale

outlist = iimagelist//ddefstring
if(access(outlist) && ddefstring!="")del(outlist,ver-)
sskyfringename=rrootname//".skyfringe.fits"	# sky+fringe image
ffringename=rrootname//".fringe.fits"		# fringe image
sskyname=rrootname//".sky.fits"			# sky-flat image

#The "go" parameter won't let the task run if you haven't loaded ccdred.
go=yes
if(!defpac("ccdred")) {go=no}
if(!deftask("imcalc")) {go=no}
if(!deftask("iterstat")) {go=no}
if(!deftask("addcomment")) {go=no}
if(go) {

#Set up log file, appending if it already exists
if(llogfile == "")
  llogfile=rrootname//".log"
print(" ", >> llogfile)

# Create fringe image step
if(getfringe){
# Do iterstat to find each image's "true" median for use in scaling
print("===> Finding image scales and weights...")
list = iimagelist
sum_exptime = 0.
scale = 1
i = 0
while (fscan(list,img) !=EOF) {
	fileroot(img,validim+)
	flat=fileroot.root//ddefstring//".fits"
	if(ddefstring!="")print(flat, >> outlist)
	i = i + 1
	force = forcescale
	imgets (img, "exptime")
	sum_exptime=sum_exptime+real(imgets.value)
	if (!force) {
		imgets (img,"SKYFRING",>&"dev$null")
		mean=real(imgets.value)
		if (mean == 0) {
			imgets (img,"ITERMED",>&"dev$null")
			valmed=real(imgets.value)
			if (valmed == 0) {
				force = yes
			}
			else {
				valmed = 1./valmed
				hedit(img,"SKYFRING",valmed,ad+,ve-,sho-,updat+)
				scale = scale + real(imgets.value)
			}
		}
		else {
			scale = scale + 1./real(imgets.value)
		}
	}
	if (force) {
		iterstat.nsigrej=5
		iterstat.maxiter=10
	iterstat(img,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print-,ver-)
		valmed = 1./iterstat.median
		hedit (img,"SKYFRING",valmed,add+,ver-,show-,update+)
		scale = scale + (1./valmed)
	}	
}
sum_exptime=sum_exptime/i

# Use this list of images to make the sky+fringe frame
print("===> Creating template sky + fringe frame...")
if(!imaccess(sskyfringename)){
  imcombine("@"//iimagelist, sskyfringename, sigma="", plfile="", combine="median", reject="ccdclip", project-, offsets="none", blank=0., scale="!SKYFRING", zero="none", weight="!SKYFRING", statsec="", lthresh=-100000, hthresh=100000, nlow=1, nhigh=1, nkeep=1, mclip+, lsigma=3, hsigma=3, rdnoise=rrdnoise, gain=ggain, snoise="0.", sigscale=0.1, pclip=-0.5, grow=0., logfile='STDOUT', >> llogfile)
} else {
  beep; print (sskyfringename//" already exists... not overwritten.")
}
hedit(sskyfringename,"exptime",sum_exptime,add+,ver-,show-,update+)
print("===> Displaying sky+fringe image for inspection.  If object residuals")
print("===> are present, you may wish to stop, create object masks, and rerun")
print("===> the task.  You can use makeobjmask on the non-defringed images to")
print("===> create object masks for them directly, or you can use shiftnadd on")
print("===> the non-defringed images, run mkobjmask on the skysubbed output,")
print("===> insert the object mask names into the non-defringed image headers,")
print("===> and then rerun this task so that the objects are masked out during")
print("===> the creation of the sky+fringe image.")
if(imdisplay)displ(sskyfringename,1)
print("===> Displaying image histogram...")
imhistog(sskyfringename,hist_type="normal",listout-,logy+)
ggoon=goon
if(!ggoon){
	print("Task aborted-- illumination correction & defringing not done.")
	if(!ssave)imdel(sskyfringename,ver-)
	if(ddefstring!="")del(outlist,ver-)
	return
	}

# "scale" is set equal to the average median of the input images.
# It is multiplied into the sky+fringe frame to keep fringe counts accurate.
scale = scale / i
imgets (sskyfringename,"SCALE",>&"dev$null")
scale2 = real(imgets.value)
if (scale2 == 0) {
	imar (sskyfringename, "*", scale, sskyfringename)
	hedit (sskyfringename, "scale", scale, add+, ver-, show-, update+)
	iterstat (sskyfringename, print+, ver-, addh+)
} else {
	beep
	print(sskyfringename//" untouched -- already scaled by factor "//scale2)
}

# Insert title into fringe image header
hedit (sskyfringename, "title", "Sky+fringe image for "//iimagelist, add+,ver-,show-,update+)

# Create sky illumination correction image
mkskycor(sskyfringename,sskyname,xboxmin=5.,xboxmax=0.25,yboxmin=5.,yboxmax=0.25)
hedit (sskyname, "title", "Sky illumination correction image for "//iimagelist, add+,ver-,show-,update+)

# Do illumination correction *on output images only*
#	imcopy("@"//iimagelist, "@"//outlist, ver-)
#	ccdproc("@"//outlist, illumcor+, illum=sskyname, zero=biasimg)

# Create fringe image
imcopy(sskyfringename, ffringename, ver-)
imar (ffringename, "-", sskyname, ffringename)
hedit (ffringename, "title", "Fringe image for "//iimagelist,add+,ver-,show-,update+)
}

# Subtract accurately scaled fringe image from each input image, if desired
if(dofringe){
  print("===> Subtracting UNSCALED fringe image from each input image...")
  list = iimagelist
  while (fscan (list, name) !=EOF) {
	  fileroot(name,validim+)
	  flat=fileroot.root//ddefstring//".fits"
	  if(!access(outlist))print(flat, >> outlist)
	  imar (name,"-",ffringename,flat,ver-,noact-,pixt="1",calct="real")
	  hedit(flat,"FRINGMED","1.",add+,ver-,show-,update+)
	  addcomment(flat,"Defringed using unscaled copy of "//ffringename,ver-)
          print(flat//" was defringed using "//ffringename)
  }
}

print("===> Wrapping up... ")
datefile = mktemp("date")
date(>>datefile)
datelist = datefile
print("finished at "//datelist, >> llogfile)
datelist = ""
del(datefile, ver-)
}
else {
	beep
	print("WARNING: one or more of the packages `noao.imred.ccdred' or")
	print("`stsdas.toolbox.imgtools' or `PHIIRS' are not loaded!")
	print("Please load them and restart this task.")
	}

if(echog)beep

end

