# optcoadd -- coadd images with sub-pixel shift options and much, much more
#
# Calls addcomment, iterstat, fileroot, filecalc, stsdas.toolbox.imgtools.imcalc
#
# 960712: created from phiirs$ircoadd.cl
# 960716: deleted internal scaling check, added subtracted-image creation
# 960723: fixed edge problem in sky-subtracted image
# 960806: made sky-subtracted image creation optional
# 960807: fixed minor bug of ccomblist2 not existing in expmap creation only
# 960812: fixed exposure map creation & created normalization map creation
# 960815: fixed bug in normalization of expmap when dividing into outimage
# 960816: changed so that statfile is used only if directly requested,
#	  and added optional insertion of variances into image headers
# 960817: fixed occasional bug in skysub + weighted-expmap normalization
# 960822: moved check for necessary tasks earlier to avoid errors
# 970425: added check for tpl image
# 970429: changed expmap -- now sets bad pixels to -99999.
#	  Also several other minor changes to bring in line with ircoadd
# 970505: changed default expmap suffix from "em" to "exp" to avoid confusion
#	  with ircoadd, whose output expmap is different than optcoadd's
# 970507: changed creation of individual expmaps to replace pixels above 
#	  -10000. with exptime -- allows for -99999. bad pixels to be excluded
# 970623: added ability to use imcombine snoise parameter 
# 980511: zeroing & scaling params double-checked (to 1%) before undoing
# 980718: changed default oextimage to fits & fixed major unzeroing bug
#
# **Could cut down on transient memory needs by finding "wtfactor" needed to
#   convert _nm image to _wt image -- stack norm, then do wtfactor & stack wt
# **Note that weights should add to 1 if taken from a file (probably matters
#   only when stacking and creating expmap, but do it always just to be safe)

procedure optcoadd(comblist,outimage,expmap,normmap,shiftlist,scaling,zeroing,weighting,magfac,stack,logfile)

string comblist	 {prompt="List of images to combine"}
string outimage	 {prompt="Output coadded image"}
string expmap    {prompt="Output exposure map (default=outimage//exp)"}
string normmap {prompt="Output normalization map (default=outimage//norm)"}
string shiftlist {prompt="File with image shifts"}
string scaling   {prompt="Imcombine scaling (none|@file|!<keyword>|etc.)"}
string zeroing   {"mode",prompt="Imcombine zero offsets (none|@file|mode|etc.)"}
string weighting {prompt="Imcombine weighting (none|@file|!<keyword>|etc.)"}
int magfac {1,min=1,max=10,prompt="Magnification factor for subpixel shifting"}
bool   stack	 {prompt="Combine images by stacking & projecting?"}
string logfile   {prompt="Name of log file"}

string normname {prompt="Header keyword for normalization factor",mode="q"}
bool   newmask=no   {prompt="Override bad pixel masks listed in image headers?"}
string badmask 	    {prompt="Bad pixel mask to use instead",mode="q"}
bool spshift=no {prompt="Do image shifts account for subpix shifting?",mode="q"}
bool prescale=no {prompt="Scale images before, not during, imcombine?",mode="q"}
bool unscale=yes {prompt="Undo scaling factors when done?",mode="q"}
real   extcoeff  {prompt="Extinction coefficient for default scalings",mode="q"}
real   airzero   {INDEF,min=0.,max=99.,prompt="Airmass to scale to? (INDEF=min. observed)",mode="q"}
bool scalecombine=no {prompt="Combine default & additional scalings?",mode="q"}
string morescalename {prompt="@filename or !keyword for additional scalings",mode="q"}
bool   putscale  {prompt="Insert scaling factors into image headers?",mode="q"}
string wordscale="SCALEFAC"    {prompt="Scaling factor header keyword",mode="q"}
bool defscale=no {prompt="Do scales include default airmass scaling?",mode="q"}
bool   prezero=no   {prompt="Do zero offseting before, not during, imcombine?",mode="q"}
bool   unzero=yes   {prompt="Undo zero offsets when done?",mode="q"}
bool   putzero=no   {prompt="Insert zero offsets into image headers?",mode="q"}
string wordzero="ZEROVAL" {prompt="Zero offset header keyword",mode="q"}
bool   putweight=no       {prompt="Insert weights into image headers?",mode="q"}
string wordweight="WEIGHT" {prompt="Weighting factor header keyword",mode="q"}
string stackimage          {prompt="Name of stacked image", mode="q"}
bool   unstack      {prompt="Delete stacked image & expmap when done?",mode="q"}
bool spscale=no {prompt="Do zero offsets account for subpix shifting?",mode="q"}
string expname="EXPTIME" {prompt="Header keyword for exposure time"}
bool   maskedstat=no     {prompt="Use maskstat to find default zeros & wts?"}
string statfile=""       {prompt="File for image statistics"}
bool   stdpars=yes       {prompt="Use std imcombine parameters for coadd?"}
bool   makeavg=no        {prompt="Make pure coadd (comb=average,rej=none)?"}
bool   flip=no           {prompt="Flip image L-R after coadd?"}
bool   cleanup=yes   {prompt="Clean up new created blkrep'd images after use?"}
bool   tpl=no        {prompt="Account for throughput correction in norm. map?"}
bool   echog        {yes,prompt="Beep when task is done?"}
pset   optcoaddpars  {prompt="Imcombine parameters"}
struct *list

begin

string sshiftfraclist, iinterp, sshiftlist, subimg, explist, sect, skysubimg
string ccomblist, bbadmask, sshiftintlist, ooutimage, zzeroing, sstackimage
string llogfile, check, zzeroname, sscalefile, sscalename, img, eexpname, blk
string s_sigma, s_plfile, s_combine, s_reject, s_offsets, s_masktyp, eexpmap
string s_scale, s_weight, s_statsec, s_expname, sscaling, intcosca, expmask
string listimshift, wwordscale, wwordzero, subimgold, expmaskold, checkmore
string ccomblist1, ccomblist2, sshiftexactlist, image, newbp, newzzeroname
string oldbp, bprotlist, file_of_zeros, list_of_zeros, checkscale, checkweight
string checkzero, newwweightname, oldzs, newzs, oldwts, newwts, wweightname
string file_of_more, mmorescalename, file_of_air, wweighting, wwordweight
string file_of_scales, list_of_scales, sstatfile, s_gain, s_rdnoise, nnormname
string nnormmap, wtimg, wtlist, wtimgold, tplimg, tplimgold, oextimage, tpltmp
string tmpimg, tmpimg2, s_snoise
real s_maskval, s_blank, s_lthresh, s_hthresh, s_nlow, s_nhigh, s_nkeep, avg, x
real s_lsigma, s_hsigma, s_pclip, s_grow, s_sigscale, sig
real effairmass, wtavg, nncombine, exp, exposure, sf, zf, testold, eexptime
real xold, yold, xnew, ynew, xshift, yshift, sscale, p, b, h, levelzero, wt
real wt_avg_exptime, unwt_avg_exptime, eextcoeff, aairzero, air, sfavg, sfmin
real sfmax, tmpzero, wait, waitold, tmpscale
int mag, mag2, j, k, tpl1, tpl2
int xmin, ymin, newx, newy, oldx, oldy, test1, test1old, test2, test2old
int sstartno, eendno, i, numpix, num, xdim, ydim, newxsize, newysize, ncoadds
bool mmakescalefile, mmakestack, go, mmorehow, pputzero, ddefscale, pputscale
bool nnewmask, uunstack, pprescale, uunscale, pputweight, sstack, getnorm, ttpl
bool sspscale, ccleanup, noappend, pprezero, uunzero, sscalecombine, sspshift
struct theadline

#for zero/scale/weight=*
# *ing	how to do it
# *name	filename, if substr(*ing,1,1)==@
# word* header keyword, if substr(*ing,1,1)==! (or @ && pre*=yes)
# pre*	do zero/scale before imcombine
# un*	undo zero/scale after imcombine

# Certain tasks/packages must be loaded for task to run.
go=yes
if(!defpac("imgtools")) {go=no}
if(!deftask("addcomment")) {go=no}
if(!deftask("fileroot")) {go=no}
if(!deftask("filecalc")) {go=no}
if(!deftask("iterstat")) {go=no}
if(go) {

# Set up parameters
ccomblist = comblist
	ccomblist1 = ccomblist
fileroot (outimage,validim+)
	ooutimage = fileroot.root
	oextimage = fileroot.extension
#	if(oextimage=="")oextimage="imh"
	if(oextimage=="")oextimage="fits"
fileroot (expmap,validim+)
	eexpmap = fileroot.root
	if(eexpmap=="")eexpmap=ooutimage//"exp"
nnormmap=normmap
	if(nnormmap=="")nnormmap=ooutimage//"norm"
sshiftexactlist = shiftlist
	sshiftintlist = sshiftexactlist
sscaling = scaling
	if(sscaling=="")sscaling="none"
	ddefscale=no
	aairzero=100.
	pprescale=no
	uunscale=no
	sscalename="_.imag_in_ary_fileJ"
	checkscale=substr(sscaling,1,1)
	if(checkscale=="!" || checkscale=="@"){
		wwordscale = substr(sscaling,2,strlen(sscaling))
		pprescale=prescale
		if(pprescale)uunscale=unscale
		if(checkscale=="@"){
			sscalename = substr(sscaling,2,strlen(sscaling))
			if(!access(sscalename)){
				ddefscale=yes
				eextcoeff=extcoeff
				aairzero=airzero
				sscalecombine=scalecombine
				if(sscalecombine){
					mmorescalename=morescalename
					checkmore=substr(mmorescalename,1,1)
  					while(checkmore!="@" && checkmore!="!"){
     print("Additional scales must have format !<header_keyword> or @filename.")
						mmorescalename=morescalename
					    checkmore=substr(mmorescalename,1,1)
					}
				}
			}
			pputscale=putscale
			if(pputscale)wwordscale=wordscale
		}
		if(!ddefscale)ddefscale=defscale
		if(ddefscale && aairzero==100.)aairzero=airzero
# if airzero=INDEF and airzero is needed later in the task, airzero=min(airmass)
                if(aairzero==INDEF && (ddefscale || (pprescale && checkscale=="@" && access(sscalename)))){
			print("...one moment please...")
                	file_of_air=mktemp(".foa")
                	hsel("@"//ccomblist1,"AIRMASS",yes, >> file_of_air)
                	list=file_of_air
                	aairzero=100.
                	while (fscan (list,air) != EOF) {
                        	aairzero=min(air,aairzero)
                	}
                	del(file_of_air,ver-,>&"dev$null")
                	list=""
        	}
	}
zzeroing = zeroing
	if(zzeroing=="")zzeroing="none"
	pprezero=no
	uunzero=no
	checkzero=substr(zzeroing,1,1)
	if(checkzero=="!" || checkzero=="@"){
		wwordzero = substr(zzeroing,2,strlen(zzeroing))
		pprezero=prezero
		if(pprezero)uunzero=unzero
		if(checkzero=="@"){
			zzeroname = substr(zzeroing,2,strlen(zzeroing))
			pputzero=putzero
			if(pputzero)wwordzero=wordzero
		}
	}
wweighting = weighting
	wwordweight=""
	if(wweighting=="")wweighting="none"
	checkweight=substr(wweighting,1,1)
	if(checkweight=="!")wwordweight=substr(wweighting,2,strlen(wweighting))
	if(checkweight=="@"){
		wweightname = substr(wweighting,2,strlen(wweighting))
		pputweight=putweight
		if(pputweight)wwordweight=wordweight
	}	
mag = magfac
	mag2=mag*mag
	blk = "br"//mag
	ccomblist2 = ccomblist//blk
	bprotlist = ccomblist//blk//"pl"
	sspscale=no
	sspshift=no
	if(mag!=1){
		if(zzeroing!="none")sspscale=spscale
		sspshift=spshift
	}
sstack=stack
sstackimage="fake_image_oog"
	if(sstack) {
		fileroot(stackimage,validim+)
		sstackimage = fileroot.root
		uunstack=unstack
	}
llogfile = logfile

nnormname=normname
	getnorm=no
	if(nnormname=="")getnorm=yes
	if(nnormname=="")nnormname="variance"
	if(wwordweight=="")wwordweight=wordweight
nnewmask = newmask
	if(nnewmask)bbadmask = badmask
eexpname = expname
sstatfile = statfile
	if(sstatfile=="")sstatfile="_nonexistentFILEname_"
ttpl = tpl
ccleanup = cleanup

# Check for bad parameter combinations and exit if necessary
if(!pprescale && sstack && checkscale=="!"){
       print("WARNING!  You can't scale during imcombine from a header keyword")
       print("when stack=yes!  (There is only one header for a stacked image.)")
       print("Use `@file' or prescale=yes instead.")
       return
}

if(!pprezero && sstack && checkzero=="!"){
	print("WARNING!  You can't take the zero offsets from a header keyword")
	print("when stack=yes! (There is only one header for a stacked image.)")
	print("Use `@filename' or prezero=yes instead.")
        return
}

if(checkweight!="@" && wweighting!="none" && sstack){
	print("WARNING! Output image exposure time would have been incorrect.")
     print("If you want to use weights and stacking, put the weights in a file")
       print("before running the task; that way the output image exposure time")
	print("will be correctly calculated.")
	return
}

wt_avg_exptime=-1.
unwt_avg_exptime=-1.

s_sigma = i_sigma	# imcombine parameters
s_plfile = i_plfile
s_combine = i_combine
s_reject = i_reject
s_masktyp = i_masktyp
s_maskval = i_maskval
s_blank = i_blank
s_statsec = i_statsec
s_expname = i_expname
s_lthresh = i_lthresh
s_hthresh = i_hthresh
s_nlow = i_nlow
s_nhigh = i_nhigh
s_nkeep = i_nkeep
s_lsigma = i_lsigma
s_hsigma = i_hsigma
s_rdnoise = i_rdnoise
s_gain = i_gain
s_snoise = i_snoise
s_sigscale = i_sigscale
s_pclip = i_pclip
s_grow = i_grow

# Set up log file, appending if it already exists.
if(llogfile == "") llogfile="log."//ooutimage
print("", >> llogfile)
print("===> PHAT.OPTCOADD:", >> llogfile)
if(mag!=1){
	print("===> Input images from "//ccomblist1//" replicated by factor "//mag//" in each direction.", >> llogfile)
	print("===> Replicated images in "//ccomblist2//" coadded w/offsets from "//sshiftexactlist//blk, >> llogfile)
} else {
	print("===> Images in "//ccomblist1//" coadded w/offsets from "//sshiftexactlist//",", >> llogfile)
}
print("===> weights="//wweighting//", stdpars="//stdpars//", makeavg="//makeavg//",", >> llogfile)
print("===> scales="//sscaling//", zero offsets="//zzeroing//",", >> llogfile)
print("===> pre-scaling="//pprescale//", un-scaling="//uunscale//", pre-zeroing="//pprezero//", un-zeroing="//uunzero//",", >> llogfile)
print("===> defscale="//ddefscale//", subpixscale="//sspscale//", subpixshift="//sspshift//",", >> llogfile)
print("===> to make image "//ooutimage//" and exposure map "//eexpmap, >> llogfile)


# PRE-SCALING AND DEFAULT SCALING FACTORS FILE CREATION 
if(pprescale){
# Prevent imcombine from scaling twice
    sscaling="none"
# If stacking, and image already exists, do not bother
#    if(sstack && access(sstackimage//".imh")){
#	print("Not bothering to pre-scale... stacked image already exists.")
#    } else {
	print("===> Multiplying by additional scaling factors...")
# In case of scalings taken from header keyword, make file_of_scales
	if(checkscale=="!"){
		file_of_scales=mktemp(".fos")
		hsel("@"//ccomblist1,wwordscale,yes, >> file_of_scales)
	}
# In case of scalings taken from file...
	if(checkscale=="@"){
#
# ...create file of DEFAULT SCALING FACTORS if it doesn't exist...
#
	    if(!access(sscalename)){
		file_of_air=mktemp(".foa")
# get airmasses
		hsel("@"//ccomblist1,"AIRMASS",yes, >> file_of_air)
# turn airmasses into scales
		list=file_of_air
		while (fscan (list,air) != EOF) {
			sf=10**(0.4*eextcoeff*(air-aairzero))
			print(sf, >> sscalename)
		}
		del(file_of_air,ver-)
# If combining default scales with other pre-existing scales...
		if(sscalecombine){
			file_of_more=mktemp(".fom")
			if(checkmore=="@"){
	 copy(substr(mmorescalename,2,strlen(mmorescalename)),file_of_more,ver-)
			}
			if(checkmore=="!"){
				hsel("@"//ccomblist1,substr(mmorescalename,2,strlen(mmorescalename)),yes, >> file_of_more)
			}
# join files of default and additional scales
			del("tmp_."//sscalename,ver-,>&"dev$null")
        		join(sscalename,file_of_more,output="tmp_."//sscalename)
			del(sscalename,ver-,>&"dev$null")	
			del(file_of_more,ver-,>&"dev$null")	
			list=""
			list="tmp_."//sscalename
# create new scales & put into scale file
			while(fscan (list,air,sf) != EOF) {
				print(air*sf, >> sscalename)
			}
			del("tmp_."//sscalename,ver-,>&"dev$null")
		}
# Finished with creation of default scalings file
	    }
	    list=""
	    file_of_scales=sscalename
# End of taking scales from list subroutine
	}
	list=""
# Allow imcombine to do sigscaling as requested even when pre-scaling
	if(s_sigscale!=0.){
		del ("_avg.file.tmp", ver-, >& "dev$null")
	    type (file_of_scales | average opt="new_sample", >> "_avg.file.tmp")
		sfavg=average.mean
		sfmin=1.E20
		sfmax=-1.E20
		list=file_of_scales
		while (fscan (list,sf) != EOF) {
			sfmin=min(sfmin,sf)
			sfmax=max(sfmax,sf)
		}
		if((sfavg-sfmin/sfavg)>=s_sigscale || (sfmax-sfavg/sfavg)>=s_sigscale)s_sigscale=0.
		del ("_avg.file.tmp", ver-, >& "dev$null")
		list=""
	}
# Do multiplicative scaling unless it has already been done
        list_of_scales=mktemp(".los")
        join(ccomblist1,file_of_scales,output=list_of_scales)
        if(checkscale=="!")del(file_of_scales,ver-)
        list=list_of_scales
        while (fscan (list,img,sf) != EOF) {
		imgets(img,"PRESCALE",>&"dev$null")
		if(imgets.value=="0"){
			imar(img,"*",sf,img,pixt="real",calct="real",ver-)
		   hedit(img,"PRESCALE",sf,add+,ver-,show-,update+,>&"dev$null")
			if(ddefscale){
				imgets(img,"AIRMASS",>&"dev$null")
	  hedit(img,"AIRMOLD",imgets.value,add+,ver-,show-,update+,>&"dev$null")
	      hedit(img,"AIRMASS",aairzero,add+,ver-,show-,update+,>&"dev$null")
			}
		} else {
			print("No action -- image "//img//" already pre-scaled by factor "//imgets.value)
		}
	}
#    }
}
list=""
# put scale factors into image headers if desired
if(checkscale=="@"){
    if(pputscale){
	del("_tmp.sscaling",ver-,>&"dev$null")
	join(ccomblist1,sscalename,out="_tmp.sscaling")
	list="_tmp.sscaling"
	while(fscan (list, img, sf) != EOF) {
		hedit(img,wwordscale,sf,add+,ver-,show-,update+,>&"dev$null")
	}
	del("_tmp.sscaling",ver-,>&"dev$null")
    }
}
list=""


# DETERMINE ZERO-LEVEL OFFSETS IF NEEDED AND INSERT INTO HEADERS IF DESIRED
if(checkzero=="@"){
    if(!access(zzeroname)){
	print("===) Finding iterative median zero offsets for the images...")
	del ("_zero.file.tmp?", ver-, >& "dev$null")
	del ("_avg.file.tmp", ver-, >& "dev$null")
# first find iterative median of each image
	if(!access(sstatfile)){
	    if(maskedstat){
		iterstat.nsigrej=5.
		iterstat.maxiter=10
		maskstat("@"//ccomblist1,"!BPM",1,lower=INDEF,upper=INDEF,iterstat+,ver-,printit+, >> "_zero.file.tmp1")
  if(sstatfile!="_nonexistentFILEname_")copy("_zero.file.tmp1", sstatfile, ver-)
            } else {
		iterstat("@"//ccomblist1,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print+,ver-, >> "_zero.file.tmp1")
  if(sstatfile!="_nonexistentFILEname_")copy("_zero.file.tmp1", sstatfile, ver-)
            }
        } else {
if(sstatfile!="_nonexistentFILEname_")print("Using statistics from"//sstatfile)
	    copy(sstatfile, "_zero.file.tmp1", ver-)
        }
# next find the overall average of the means
	fields ("_zero.file.tmp1", 5, >> "_zero.file.tmp2")
	type ("_zero.file.tmp2" | average opt="new_sample", >> "_avg.file.tmp")
# now compute the offset of each image from the overall average mean
	list="_avg.file.tmp"
	while (fscan (list, avg, sig, num) !=EOF) {
	lintran("_zero.file.tmp2",min=2,x2=avg,yfield=1,angle=180, >> zzeroname)
	}
	del ("_zero.file.tmp?", ver-)
	del ("_avg.file.tmp", ver-)
    }
# now insert into image headers if desired
    if(pputzero) {
	if(access("_tmp.zzeroing"))del("_tmp.zzeroing",ver-)
	join(ccomblist1,zzeroname,out="_tmp.zzeroing")
	list=""
	list="_tmp.zzeroing"
	while (fscan (list, img, zf) !=EOF) {
     		hedit(img,wwordzero,zf,add+,ver-,show-,update+)
	}
	del("_tmp.zzeroing",ver-,>&"dev$null")
    }
} 
list=""


# DETERMINE WEIGHTS IF NEEDED AND ADD TO IMAGE HEADERS IF DESIRED
if(checkweight=="@"){
    if(!access(wweightname)){
print("===) Creating file of optimal exptime/rms**2 weights for the images...")
        del ("_weight.file.tmp?", ver-, >& "dev$null")
        del ("_avg.file.tmp", ver-, >& "dev$null")
# first find iterative rms of each image
	#hsel("@"//ccomblist1,"PRESCALE",yes, >> "_weight.file.tmp9")
	#count("_weight.file.tmp9") | scan (i,j,k)
	#if(j!=0){
	#beep
	#print("WARNING! At least one image in "//ccomblist//" was pre-scaled;")
	#print("thus, the image weights being calculated will be erroneous.")
	#}
	if(!access(sstatfile)){
	    if(maskedstat){
		iterstat.nsigrej=5.
		iterstat.maxiter=10
		if(!access("_weight.file.tmp1"))maskstat("@"//ccomblist1,"!BPM",1,lower=INDEF,upper=INDEF,iterstat+,ver-,printit+, >> "_weight.file.tmp1")
if(sstatfile!="_nonexistentFILEname_")copy("_weight.file.tmp1", sstatfile, ver-)
            } else {
		if(!access("_weight.file.tmp1"))iterstat("@"//ccomblist1,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print+,ver-, >> "_weight.file.tmp1")
if(sstatfile!="_nonexistentFILEname_")copy("_weight.file.tmp1", sstatfile, ver-)
            }
        } else {
	    copy (sstatfile, "_weight.file.tmp1", ver-)
        }
        fields ("_weight.file.tmp1", 3, >> "_weight.file.tmp2")
# get the exptimes for the images
	hsel("@"//ccomblist1,eexpname,yes, >> "_weight.file.tmp0")
        join("_weight.file.tmp2","_weight.file.tmp0",out="_weight.file.tmp3")
# compute the weights = exptime./(rms**2.)
        filecalc ("_weight.file.tmp3", "$2/($1*$1)", calctype="real", >> "_weight.file.tmp4")
# find the sum of the weights
        type ("_weight.file.tmp4" | average opt="new_sample",>> "_avg.file.tmp")
# normalize by the sum of the weights
	filecalc ("_weight.file.tmp4", "$1/"//average.sum, calctype="real", >> wweightname)
        del ("_weight.file.tmp?", ver-)
        del ("_avg.file.tmp", ver-)
    }
    if(pputweight){
	if(access("_tmp.wweighting"))del("_tmp.wweighting",ver-)
	join(ccomblist1,wweightname,out="_tmp.wweighting")
	list="_tmp.wweighting"
	while (fscan (list, img, wt) !=EOF) {
     		hedit(img,wwordweight,wt,add+,ver-,show-,update+)
	}
	del("_tmp.wweighting",ver-)
    }
}
list=""


# IF DESIRED, DETERMINE NORMALIZATION FACTORS AND ADD TO IMAGE HEADERS
if(getnorm){
print("===) Finding normalization factors (variances) and putting into image headers...")
        del ("_norm.file.tmp?", ver-, >& "dev$null")
# first find iterative rms of each image
	if(!access(sstatfile)){
	    if(maskedstat){
		iterstat.nsigrej=5.
		iterstat.maxiter=10
		if(!access("_norm.file.tmp1"))maskstat("@"//ccomblist1,"!BPM",1,lower=INDEF,upper=INDEF,iterstat+,ver-,printit+, >> "_norm.file.tmp1")
if(sstatfile!="_nonexistentFILEname_")copy("_norm.file.tmp1", sstatfile, ver-)
            } else {
		if(!access("_norm.file.tmp1"))iterstat("@"//ccomblist1,nsigrej=5.,maxiter=10,lower=INDEF,upper=INDEF,print+,ver-, >> "_norm.file.tmp1")
if(sstatfile!="_nonexistentFILEname_")copy("_norm.file.tmp1", sstatfile, ver-)
            }
        } else {
	    copy (sstatfile, "_norm.file.tmp1", ver-)
        }
        fields ("_norm.file.tmp1", 3, >> "_norm.file.tmp2")
# compute the normalization factor = (rms**2.)
    filecalc ("_norm.file.tmp2", "$1*$1", calctype="real", >> "_norm.file.tmp4")
	if(access("_tmp.norm"))del("_tmp.norm",ver-)
	join(ccomblist1,"_norm.file.tmp4",out="_tmp.norm")
	list="_tmp.norm"
	while (fscan (list, img, wt) !=EOF) {
     		hedit(img,nnormname,wt,add+,ver-,show-,update+)
	}
	del("_tmp.norm",ver-)
        del ("_norm.file.tmp?", ver-)
}
list=""


# SUBPIXEL SHIFTING
noappend=no
# First expand images if subpixel shifting is desired
if (mag!=1){
	if(!sspshift)sshiftintlist=sshiftexactlist//blk
# If output image list already exists, do not append to it
	if (access(ccomblist2)){
        	noappend=yes
        print("WARNING! Output image list "//ccomblist2//" already exists.")
        print("Any images listed therein will be created if they do not exist,")
        print("but no image names will be appended to this preexisting file.")
	}
	print("===) Expanding images by factor "//mag//" in both directions...")
	list=ccomblist1
# Go through image list
	while (fscan (list, img) !=EOF) {
		fileroot(img,validim+)
		image=fileroot.root
# Do the magnification as long as the image does not already exist or has not
# already been rotated/magnified
		if(!access(image//blk//".fits")){
        	    imgets(image,"ROTATED",>&"dev$null")
        	    if(imgets.value=="0"){
# Divide by mag squared to conserve total number of counts in input image.
	       imar(image,"/",mag2,"_"//blk//image,pixt="real",calc="real",ver-)
			blkrep("_"//blk//image,image//blk,mag,mag)
			imdel("_"//blk//image,ver-)
			if(!noappend)print (image//blk, >> ccomblist2)
# Now replicate the bad pixel mask, if not already done.
        		imgets(image//blk,"BPMASK",>&"dev$null")
        	      if(imgets.value=="0")imgets(image//blk,"BPM",>&"dev$null")
			oldbp=imgets.value
			newbp=blk//oldbp
			if(!access(newbp)){
				blkrep(oldbp,newbp,mag,mag)
				print(newbp//".pl", >> bprotlist)
			}
# Insert the replicated bad pixel mask in the image header
			hedit(image//blk,"BPMASK",newbp,add+,ver-,show-,update+)
			hedit(image//blk,"BPM",newbp,add+,ver-,show-,update+)
                    } else {
# If input image already rotated, only print a warning.
	    print("WARNING! "//image//".fits already rotated; not rotated again")
                    }
                } else {
# If output image already exists, only print a warning.
print("WARNING! "//image//blk//".fits already exists; not overwritten")
                }
# Next image
	}
#
# If output subpixel shifts file doesn't exist, create it
	if(!access(sshiftintlist)) {
	        print("===> Calculating shifts for expanded images...")
		if (access("tmp"//sshiftintlist))del("tmp"//sshiftintlist,ver-)
		list=""
		list=sshiftexactlist
# Convert old shifts to new
		while (fscan (list,xold,yold) != EOF) {
			xnew=mag*xold
			ynew=mag*yold
			print (xnew, ynew, >> "tmp"//sshiftintlist)
		}
# Round off to integer shifts (in units of new pixels, natch).
   filecalc("tmp"//sshiftintlist,"$1;$2",format="%4.0f %4.0f", >> sshiftintlist)
		del("tmp"//sshiftintlist,ver-)
# If output subpixel shifts file does exist, just print warning
	} else {
		if(sshiftintlist==sshiftexactlist//blk)print("WARNING! "//sshiftintlist//" already exists; not overwritten")
	}
# Correction of zero offsets for lower flux/pixel in blkrep'd images, if needed
	if(!sspscale) {
# Zero offsets from list
		if(checkzero=="@") {
		    if(!access(zzeroname//blk)) {
			filecalc(zzeroname,"$1/"//mag2,format="%9.7f", >> zzeroname//blk)
			newzzeroname=zzeroname//blk
			zzeroname=newzzeroname
			zzeroing="@"//newzzeroname
		    }
		}
# Zero offsets from header keyword
		if(checkzero=="!"){
			oldzs=mktemp(".oz")
			newzs=mktemp(".nz")
			hsel("@"//ccomblist1,wwordzero,yes, >> oldzs)
		        filecalc(oldzs,"$1/"//mag2,format="%9.7f", >> newzs)
			join(ccomblist2,oldzs,output=newzs)
			list=""
			list=newzs
			while (fscan (list,img,levelzero) != EOF) {
			  hedit(img,wwordzero,levelzero,add+,ver-,show-,update+)
			}
			del(oldzs,ver-)
			del(newzs,ver-)
		}
	}
# End of subpixel shift option
}
list=""


# INSERT NEW ZERO OFFSETS INTO REPLICATED IMAGE HEADERS IF DESIRED.
if(access(ccomblist2) && checkzero=="@"){
    if(pputzero) {
	if(!sspscale && access(zzeroname//blk))zzeroname=zzeroname//blk
	del("_tmp.zzeroing",ver-,>&"dev$null")
	join(ccomblist2,zzeroname,out="_tmp.zzeroing")
	list="_tmp.zzeroing"
	while (fscan (list, img, zf) !=EOF) {
		hedit(img,wwordzero,zf,add+,ver-,show-,update+)
	}
	del("_tmp.zzeroing",ver-,>&"dev$null")
    }
}
list=""


# PRE-ZEROING: INCORPORATE ZERO OFFSETS BEFORE IMCOMBINE IF DESIRED
if(pprezero){
	print("===> Doing zero offsetting prior to imcombine...")
# Prevent imcombine from zeroing twice
        zzeroing="none"
# If list of images to coadd doesn't exist, create it!
	if (!access(ccomblist2))copy(ccomblist1,ccomblist2,ver-)
# In case of zero offsets taken from header keyword, make file_of_zeros
	if(checkzero=="!"){
		file_of_zeros=mktemp(".foz")
		hsel("@"//ccomblist2,wwordzero,yes, >> file_of_zeros)
	}
	if(checkzero=="@")file_of_zeros=zzeroname
	list_of_zeros=mktemp(".loz")
	join(ccomblist2,file_of_zeros,output=list_of_zeros)
	if(checkzero=="!")del(file_of_zeros,ver-)
# If stacking, and image already exists, do not bother
        if(sstack && access(sstackimage//".fits")){
	    print("Not bothering to pre-zero... stacked image already exists.")
        } else {
# Do zero offsetting unless it has already been done
	    list=list_of_zeros
	    while (fscan (list,img,levelzero) != EOF) {
		    imgets(img,"PREZERO",>&"dev$null")
		    if(imgets.value=="0"){
	    imar(img,"+",levelzero,img,pixt="real",calc="real",ver-)
	    hedit(img,"PREZERO",levelzero,add+,ve-,sho-,updat+,>&"dev$null")
		    } else {
   			    print("No action -- image "//img//" already zero-offset by "//imgets.value)
		    }
	    }
	}
}
list=""


# COADD images
# If output coadded image doesn't exist, create it!
if (!access(ooutimage//"."//oextimage)){
# If list of images to coadd doesn't exist, create it!
if (!access(ccomblist2))copy(ccomblist1,ccomblist2,ver-)
if (sstack){
# If output stacked image doesn't exist, create it!
	if (!access(sstackimage//"."//oextimage)){
		print("===) Stacking the images...")
		imstack("@"//ccomblist2,sstackimage,title="Stacked images listed in "//ccomblist2)
		}
	if(nnewmask) {
		hedit(sstackimage,"BPM",bbadmask,add+,ver-,show-,update+)
	}
	print("===) Combining the images...")
	if (makeavg){
	imcombine(sstackimage, ooutimage, sigma=s_sigma, plfile=s_plfile, combine="average", reject="none", project+, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=s_blank, scale=sscaling, zero=zzeroing, weight=wweighting, statsec=s_statsec, expname=s_expname, lthresh=s_lthresh, hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=s_rdnoise, gain=s_gain, snoise="0.", sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
	}
	else{
		if (stdpars) {
		imcombine(sstackimage, ooutimage, sigma="", plfile="", combine="average", reject="ccdclip", project+, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=0., scale=sscaling, zero=zzeroing, weight=wweighting, statsec="", expname=eexpname, lthresh=INDEF, hthresh=INDEF, nlow=1., nhigh=1., nkeep=1., mclip+, lsigma=5., hsigma=5., rdnoise=s_rdnoise, gain=s_gain, snoise="0.", sigscale=s_sigscale, pclip=-0.5, grow=0., logfile='STDOUT', >> llogfile)
		}
		else {
		imcombine(sstackimage, ooutimage, sigma=s_sigma, plfile=s_plfile, combine=s_combine, reject=s_reject, project+, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=s_blank, scale=sscaling, zero=zzeroing, weight=wweighting, statsec=s_statsec, expname=s_expname, lthresh=s_lthresh, hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=s_rdnoise, gain=s_gain, snoise=s_snoise, sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
		}
	}
# Stacked images only: compute correct UNWEIGHTED average exposure time
	del("tmp_"//ccomblist2,ver-,>&"dev$null")
      hsel ("@"//ccomblist2,eexpname,yes, >> "tmp_"//ccomblist2)
    type ("tmp_"//ccomblist2) | average (opt="new_sample") | scan (eexptime,p,b)
      hedit(ooutimage,eexpname,eexptime,add+,ver-,show-,update+)
	del("tmp_"//ccomblist2,ver-,>&"dev$null")
#
if(uunstack)imdel(sstackimage,ver-)
}
else{
	if(nnewmask) {
		list=ccomblist2
		while (fscan (list, img) !=EOF) {
			imgets(img,"BPMASK",>&"dev$null")
			if(imgets.value=="0") {
			       imgets(img,"BPM",>&"dev$null")
			hedit(img,"BPMASK",imgets.value,add+,ver-,show-,update+)
			       hedit(img,"BPM",bbadmask,add+,ver-,show-,update+)
			}
			else {
			       hedit(img,"BPM",bbadmask,add+,ver-,show-,update+)
			}
		}
	}
	print("===) Combining the images...")
	if (makeavg){
	imcombine("@"//ccomblist2, ooutimage, sigma=s_sigma, plfile=s_plfile, combine="average", reject="none", project-, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=s_blank, scale=sscaling, zero=zzeroing, weight=wweighting, statsec=s_statsec, expname=s_expname, lthresh=s_lthresh, hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=s_rdnoise, gain=s_gain, snoise=s_snoise, sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
		}
	else{
		if (stdpars) {
		imcombine("@"//ccomblist2, ooutimage, sigma="", plfile="", combine="average", reject="ccdclip", project-, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=0., scale=sscaling, zero=zzeroing, weight=wweighting, statsec="", expname=eexpname, lthresh=INDEF, hthresh=INDEF, nlow=1., nhigh=1., nkeep=1., mclip+, lsigma=5., hsigma=5., rdnoise=s_rdnoise, gain=s_gain, snoise="0.", sigscale=0.1, pclip=-0.5, grow=0., logfile='STDOUT', >> llogfile)
		}
		else {
		imcombine("@"//ccomblist2, ooutimage, sigma=s_sigma, plfile=s_plfile, combine=s_combine, reject=s_reject, project-, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=s_blank, scale=sscaling, zero=zzeroing, weight=wweighting, statsec=s_statsec, expname=s_expname, lthresh=s_lthresh, hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=s_rdnoise, gain=s_gain, snoise="0.", sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
		}
	}
}
list=""

# Put original BPM back if necessary
if(nnewmask) {
	list=ccomblist2
	while (fscan (list, img) !=EOF) {
		imgets(img,"BPMASK",>&"dev$null")
		hedit(img,"BPM",imgets.value,add+,ver-,show-,update+)
	}
}
list=""

# Header parameters for the coadded image
hedit(ooutimage,"title","Coadded image from "//ccomblist2//" and "//sshiftintlist,ver-,show-,update+)
time | scan (theadline)
addcomment(ooutimage,"phat.optcoadd: "//theadline,ver-)
addcomment(ooutimage,"Coadded from "//ccomblist2//" and "//sshiftintlist,ver-)
addcomment(ooutimage,"makeavg="//makeavg//", zero shifts from "//zzeroing,ver-)
addcomment(ooutimage,"scaling factors from "//sscaling//", and stdpars="//stdpars,ver-)


# AIRMASS
# Set airmass of coadded image to effective airmass if default scaling !used
if(!ddefscale){
	del("_airmass.tmp?",ver-,>&"dev$null")
	hsel("@"//ccomblist2, "airmass", yes, >> "_airmass.tmp1")
	type("_airmass.tmp1" | average opt="new_sample", >> "_airmass.tmp2")
	effairmass=real(average.mean)
	hedit(ooutimage,"airmass",(effairmass),add+,ver-,show-,update+)
	del("_airmass.tmp?",ver-,>&"dev$null")
# Set airmass of coadded image to reference airmass if default scaling was used
} else {
	hedit(ooutimage,"airmass",(aairzero),add+,ver-,show-,update+)
}


# EXPOSURE NORMALIZATION -- normalize coadded image to total exposure time.
# If weighting was done w/a stacked image, first find correct WEIGHTED exptime
del ("_exptime.tmp?", ver-, >& "dev$null")
if(sstack && wweighting!="none"){
# First get individual image exposure times
	del ("_exptime.tmp?", ver-, >& "dev$null")
	hsel("@"//ccomblist2,eexpname,yes, >> "_exptime.tmp1")
# Join individual image exptimes with weights
	join("_exptime.tmp1", wweightname, output="_exptime.tmp3")
# Calculate individual image weighted exposure times
	filecalc ("_exptime.tmp3", "$1*$2", calctype="real", >> "_exptime.tmp4")
# Calculate correct weighted exposure time
        type ("_exptime.tmp4" | average opt="new_sample",>> "_exptime.tmp5")
# Insert correct weighted exposure time into image header
	hedit(ooutimage,eexpname,average.sum,add+,ver-,show-,update+)
}
# First find total exposure time of input images
hsel("@"//ccomblist2,eexpname,yes, >> "_exptime.tmp6")
type("_exptime.tmp6" | average opt="new_sample", >> "_exptime.tmp7")
del ("_exptime.tmp?", ver-, >& "dev$null")
unwt_avg_exptime=real(average.mean)
eexptime=real(average.sum)
# Get output exposure time of coadded image
imgets(ooutimage,eexpname)
wt_avg_exptime=real(imgets.value)
# Insert total exposure time into coadded image header
hedit(ooutimage,eexpname,eexptime,add+,ver-,show-,update+)
# Multiply coadded image by (total_exposure_time/output_exposure_time)
imar(ooutimage, "*", eexptime/wt_avg_exptime, ooutimage)
print("", >> llogfile)
print("Coadded image multiplied by "//eexptime/wt_avg_exptime//" to bring its flux", >> llogfile)
print("in line with the total exposure time of "//eexptime//" s.", >> llogfile)


# CREATE WEIGHTED EXPOSURE MAP
# Create exposure maps if the output coadded-image exposure map does not exist.
# Routine allows for images of different exposure times, sizes, and BPMs, while
# creating as few images as possible consistent with meeting those criteria.
explist=""
if(!access(eexpmap//"."//oextimage)){
# Don't recreate exposure maps to be stacked if stacked expmap image exists
    if(!sstack || (sstack && !access("exp"//sstackimage//"."//oextimage))) {
        print("===) Creating exposure maps...")
	subimgold="_oops"
	testold=0.
	test1old=0
	test2old=0
	expmaskold=""
# If list of images to coadd doesn't exist, create it!
        if (!access(ccomblist2))copy(ccomblist1,ccomblist2,ver-)
	list=ccomblist2
	explist=ccomblist2//"exp"
	if(access(explist))del(explist,ver-,>&"dev$null")
	while (fscan (list, img) !=EOF) {
# get image parameters: xsize, ysize, exptime, badpixelmask
		fileroot(img,validim+)
                imgets(img,"i_naxis1")
		test1=int(imgets.value)
		imgets(img,"i_naxis2")
 		test2=int(imgets.value)
		imgets(img,eexpname)
		exposure=real(imgets.value)
		imgets(img,"BPMASK")
		expmask=str(imgets.value)
# if any of those parameters differ from the previous image, need to create a 
# new exposure map for eventual coadding.
		if(test1!=test1old || test2!=test2old || exposure!=testold || expmask!=expmaskold) {
			subimg = mktemp(fileroot.root)
			print (subimg, >> explist)
#                        imcalc(img,subimg,"im1*0+"//exposure,ver-)
                        imcopy(img,subimg,ver-)
                        imreplace(subimg,exposure,lower=-10000.,upper=INDEF)
        	if(nnewmask)hedit(subimg,"BPM",bbadmask,add+,ver-,show-,update+)
			test1old=test1
			test2old=test2
			testold=exposure
			expmaskold=expmask
			subimgold=subimg
		}
		else {
			print (subimgold, >> explist)
		}
	}
    }
# Create coadded-image exposure map
	print("===> Creating coadded-image exposure map...")
	if(sstack) {
		if(!access("exp"//sstackimage//"."//oextimage)) {
			print("===> Stacking exposure maps...")
			imstack("@"//explist,"exp"//sstackimage,title="Stacked exposure maps listed in "//explist)
		}
		else {
	   print("Stacked exposure map image already exists-- not overwritten.")
		}
		if(nnewmask) {
		hedit("exp"//sstackimage,"BPM",bbadmask,add+,ver-,show-,update+)
		}
		imcombine("exp"//sstackimage, eexpmap, sigma="", plfile="", combine="average", reject="none", project+, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=s_blank, scale="none", zero="none", weight=wweighting, statsec=s_statsec, expname=s_expname, lthresh=0., hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=0., gain=1., snoise="0.", sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
		if(nnewmask) {
			imgets("exp"//sstackimage,"BPMASK",>&"dev$null")
		hedit("exp"//sstackimage,"BPM",imgets.value,add+,ve-,sho-,upda+)
		}
		if(uunstack)imdel("exp"//sstackimage,ver-)
	}
	else {
		imcombine("@"//explist, eexpmap, sigma="", plfile="", combine="average", reject="none", project-, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=s_blank, scale="none", zero="none", weight=wweighting, statsec=s_statsec, expname=s_expname, lthresh=0., hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=0., gain=1., snoise="0.", sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
	}
# following line had been commented out; replaced with 2nd line following
#       imreplace(eexpmap, -99999., lower=INDEF, upper=-10000.)
        imreplace(eexpmap, -99999., lower=INDEF, upper=-1.)
        hedit(eexpmap,"title","Weighted exposure map for "//ooutimage,add+,ve-,sho-,updat+)
	imgets(eexpmap,"ncombine")
	nncombine=int(imgets.value)
	if(explist!=""){
		imdel("@"//explist, ver-, >& "dev$null")
		del(explist, ver-, >& "dev$null")
	}

} else {
	print("Output exposure map already exists-- new one not created.")
 print("Output exposure map already exists-- new one not created.", >> llogfile)
}


# SUBTRACT constant background from image and DIVIDE by normalized 
# WEIGHTED EXPOSURE MAP to get image with globally constant exposure time
if(!access(ooutimage//"mask.pl")){
# create mask image with border pixels = 0 whether originally 0 or -99999
	imar(eexpmap, "+", "99999.", "j1"//eexpmap, noact-, ver-)
	imar(eexpmap, "*", "j1"//eexpmap, "j2"//eexpmap, noact-, ver-)
	imreplace("j2"//eexpmap, 1., lower=0.01, upper=INDEF)
	imcopy("j2"//eexpmap, ooutimage//"mask.pl", ver-)
	imdel("j1"//eexpmap, ver-)
	imdel("j2"//eexpmap, ver-)
}
maskstat(ooutimage, ooutimage//"mask.pl", 1, lower=INDEF, upper=INDEF, iterstat+, ver-, printit+, >> llogfile)
imar(ooutimage, "-", iterstat.mean, ooutimage, ver-, noact-)
#
imar(eexpmap,"/",wt_avg_exptime,"j1"//eexpmap,ver-,noact-)
imar("j1"//eexpmap,"/",ooutimage//"mask.pl","j2"//eexpmap,ver-,noact-,divze=-99999.)
imar(ooutimage,"/","j2"//eexpmap//"."//oextimage,ooutimage,ver-,noact-,divze=-99999.)
imdel("j?"//eexpmap//"."//oextimage,ver-)
#
print("Weighted exposure map normalized by "//wt_avg_exptime, >> llogfile)
print("and divided into output coadded image.", >> llogfile)

# FLIP image over to correct orientation if necessary (N is up, E is left)
if(flip){
  imcopy(ooutimage//"[-*,*]", ooutimage, ver-)
  addcomment(ooutimage,"Image has been flipped L-R from orig. orientation",ver-)
  print("WARNING! Image has been flipped L-R! (i.e. imcopy orig[-*,*] new)")
}

} else {
	print("Output coadded image already exists-- coadding not done.")
  print("Output coadded image already exists-- coadding not done.", >> llogfile)
}


# NORMALIZATION MAPS
# Create normalization maps if the output normalization map does not exist.
# Routine allows for images of different exposure times, sizes, and BPMs, while
# creating as few images as possible consistent with meeting those criteria.
explist=""
wtlist=""
if(!access(nnormmap//"."//oextimage)){
# Don't recreate normalization maps to be stacked if stacked norm. image exists
    if(!sstack || (sstack && !access("norm"//sstackimage//"."//oextimage))) {
        print("===) Creating normalization maps...")
	subimgold="_oops"
	wtimgold="_oops"
	waitold=0.
	testold=0.
	test1old=0
	test2old=0
	expmaskold=""
	tplimgold=""
# If list of images to coadd doesn't exist, create it!
        if (!access(ccomblist2))copy(ccomblist1,ccomblist2,ver-)
	list=ccomblist2
	explist=ccomblist2//"norm"
	wtlist=ccomblist2//"wt"
	if(access(explist))del(explist,ver-,>&"dev$null")
	if(access(wtlist))del(wtlist,ver-,>&"dev$null")
	while (fscan (list, img) !=EOF) {
# get image parameters: xsize, ysize, weight, norm.factor, badpixelmask
		fileroot(img,validim+)
                imgets(img,"i_naxis1")
		test1=int(imgets.value)
		imgets(img,"i_naxis2")
 		test2=int(imgets.value)
		imgets(img,wwordweight)
		wait=real(imgets.value)
		imgets(img,nnormname)
		exposure=real(imgets.value)
		imgets(img,"BPMASK")
		expmask=str(imgets.value)
		imgets(img,"TPLIMG",>&"dev$null")
		tplimg=str(imgets.value)
# if any of those parameters differ from the previous image, need to create a 
# new normalization map for eventual coadding.
		if(test1!=test1old || test2!=test2old || wait!=waitold || exposure!=testold || expmask!=expmaskold || tplimg!=tplimgold) {
			tmpimg = mktemp("_tmp")
			tmpimg2 = mktemp("_tmp2")
			subimg = mktemp("_nm")
			wtimg = mktemp("_wt")
			print (subimg, >> explist)
			print (wtimg, >> wtlist)
# want to end up with  wtimg = wait or -99999.; 
#		      subimg = exposure * (tpl**2.) * wait**2. or -99999.;
# make tmpimg -99999 where img < -10000, 0 elsewhere
                        imcopy(img,tmpimg,ver-)
			imreplace(tmpimg,0.,lower=-10000.,upper=INDEF)
			imreplace(tmpimg,-99999.,lower=INDEF,upper=-9999.)
# make wtimg 0 where img < -10000, "wait" elsewhere
			imar(tmpimg,"+",99999.,wtimg,ver-)
                        imreplace(wtimg,wait,lower=10000.,upper=INDEF)
# tpl
                        if(ttpl && access(tplimg)) {
                                tpltmp=mktemp("tpltmp")
                                imgets(tplimg,"i_naxis1")
                                tpl1=int(imgets.value)
                                imgets(tplimg,"i_naxis2")
                                tpl2=int(imgets.value)
# if tpl image is not same size as actual image, assume the image has been
# copied to subsection of bigger image starting at (1,1), and that the rest of
# the image is -99999 (thus no imar(img,"-",img,subimg) needed.  (ie 61"+90")
                               if(tpl1!=test1 || tpl2!=test2) {
                                        imcopy(img,subimg,ver-)
# tpltmp = tmpimg + exposure * (wtimg / tplimg)**2.
#imcalc(tmpimg//","//wtimg//","//tplimg,tpltmp,"im1+"//str(exposure)//"*(im2/im3)**2.",ver-)
	 imcalc(wtimg//","//tplimg,tmpimg2,str(exposure)//"*(im1/im2)**2.",ver-)
				imar(tmpimg, "+", tmpimg2, tpltmp, ver-, noact-)
	   imcopy(tpltmp, subimg//"[1:"//str(tpl1)//",1:"//str(tpl2)//"]", ver-)
                                        imdel(tmpimg2, ver-)
                                        imdel(tpltmp, ver-)
                                } else {
# subimg = tmpimg + exposure * (wtimg / tplimg)**2.
#imcalc(tmpimg//","//wtimg//","//tplimg,subimg,"im1+"//str(exposure)//"*(im2/im3)**2.",ver-)
	 imcalc(wtimg//","//tplimg,tmpimg2,str(exposure)//"*(im1/im2)**2.",ver-)
				     imar(tmpimg,"+",tmpimg2,subimg,ver-,noact-)
					imdel(tmpimg2,ver-)
                                }
# no tpl
			} else {
# subimg = tmpimg + exposure * wtimg**2. (exposure=exptime or variance)
	  		  imcalc(wtimg,tmpimg2,"(im1**2.)*"//str(exposure),ver-)
	  			imar(tmpimg,"+",tmpimg2,subimg,ver-,noact-)
				imdel(tmpimg2,ver-)
                        }
# now make wtimg -99999 where img < -10000, "wait" elsewhere
#			tmpimg2 = mktemp("_tmp2")
			imren(wtimg,tmpimg2,ver-)
			imar(tmpimg2,"+",tmpimg,wtimg,ver-)
			imdel(tmpimg2,ver-)
			imdel(tmpimg,ver-)
        	 if(nnewmask)hedit(wtimg,"BPM",bbadmask,add+,ver-,show-,update+)
        	if(nnewmask)hedit(subimg,"BPM",bbadmask,add+,ver-,show-,update+)
			test1old=test1
			test2old=test2
			tplimgold=tplimg
			waitold=wait
			testold=exposure
			expmaskold=expmask
			subimgold=subimg
			wtimgold=wtimg
		}
		else {
			print (subimgold, >> explist)
			print (wtimgold, >> wtlist)
		}
	}
    }

# Create coadded-image normalization map
# First create variance map
	print("===> Creating coadded-image normalization map...")
	if(sstack) {
		if(!access("norm"//sstackimage//"."//oextimage)) {
			print("===> Stacking variance maps...")
			imstack("@"//explist,"norm"//sstackimage,title="Stacked normalization maps listed in "//explist)
		}
		else {
      	   print("Stacked variance map image already exists-- not overwritten.")
		}
		if(nnewmask) {
	        hedit("norm"//sstackimage,"BPM",bbadmask,add+,ver-,show-,updat+)
		}
	    if(access("junknorm."//oextimage))imdel("junknorm."//oextimage,ver-)
		if(access(nnormmap//"pl.pl"))imdel(nnormmap//"pl.pl",ver-)
		imcombine("norm"//sstackimage, "junknorm", sigma="", plfile=nnormmap//"pl", combine="average", reject="none", project+, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=0., scale="none", zero="none", weight="none", statsec=s_statsec, expname=s_expname, lthresh=0., hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=0., gain=1., snoise="0.", sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
		if(nnewmask){
			imgets("norm"//sstackimage,"BPMASK",>&"dev$null")
		hedit("norm"//sstackimage,"BPM",imgets.value,ad+,ve-,sho-,upda+)
		}
		if(uunstack)imdel("norm"//sstackimage,ver-)
	}
	else {
	    if(access("junknorm."//oextimage))imdel("junknorm."//oextimage,ver-)
		if(access(nnormmap//"pl.pl"))imdel(nnormmap//"pl.pl",ver-)
		imcombine("@"//explist, "junknorm", sigma="", plfile=nnormmap//"pl", combine="average", reject="none", project-, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=0., scale="none", zero="none", weight="none", statsec=s_statsec, expname=s_expname, lthresh=0., hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=0., gain=1., snoise="0.", sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
	}
	if(explist!=""){
		imdel("@"//explist, ver-, >& "dev$null")
		del(explist, ver-, >& "dev$null")
	}

# Now create weight map
	if(sstack) {
		if(!access("wt"//sstackimage//"."//oextimage)) {
			print("===> Stacking weight maps...")
			imstack("@"//wtlist,"wt"//sstackimage,title="Stacked weight maps listed in "//wtlist)
		}
		else {
      	     print("Stacked weight map image already exists-- not overwritten.")
		}
		if(nnewmask) {
	          hedit("wt"//sstackimage,"BPM",bbadmask,add+,ver-,show-,updat+)
		}
	if(access("junkweight."//oextimage))imdel("junkweight."//oextimage,ver-)
		imcombine("wt"//sstackimage, "junkweight", sigma="", plfile="", combine="average", reject="none", project+, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=0., scale="none", zero="none", weight="none", statsec=s_statsec, expname=s_expname, lthresh=0., hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=0., gain=1., snoise="0.", sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
		if(nnewmask){
			imgets("wt"//sstackimage,"BPMASK",>&"dev$null")
		hedit("wt"//sstackimage,"BPM",imgets.value,add+,ve-,sho-,upda+)
		}
		if(uunstack)imdel("wt"//sstackimage,ver-)
	}
	else {
	if(access("junkweight."//oextimage))imdel("junkweight."//oextimage,ver-)
		imcombine("@"//wtlist, "junkweight", sigma="", plfile="", combine="average", reject="none", project-, offsets=sshiftintlist, masktype=s_masktyp, maskvalue=s_maskval, blank=0., scale="none", zero="none", weight="none", statsec=s_statsec, expname=s_expname, lthresh=0., hthresh=s_hthresh, nlow=s_nlow, nhigh=s_nhigh, nkeep=s_nkeep, mclip+, lsigma=s_lsigma, hsigma=s_hsigma, rdnoise=0., gain=1., snoise="0.", sigscale=s_sigscale, pclip=s_pclip, grow=s_grow, logfile='STDOUT', >> llogfile)
	}
	if(wtlist!=""){
		imdel("@"//wtlist, ver-, >& "dev$null")
		del(wtlist, ver-, >& "dev$null")
	}

# Final steps to create normalization image
	imgets("junknorm","ncombine")
	nncombine=int(imgets.value)
  if(access(nnormmap//"num."//oextimage))imdel(nnormmap//"num."//oextimage,ver-)
	imar(nnormmap//"pl","-",nncombine,nnormmap//"num.fits")
	imar(nnormmap//"num.fits","*",-1.,nnormmap//"num.fits")
	imar("junknorm","*",nnormmap//"num.fits","junknorm",ver-,noact-)
        imar("junkweight","*",nnormmap//"num.fits","junkweight",ver-,noact-)
#imar("junkweight","*",eexpmap,"junkweight",ver-,noact-)
#imar("junkweight","*","junkweight","junkweight",ver-,noact-)
#imar("junkweight","/","junknorm",nnormmap,ver-,noact-)
 	imcalc("junkweight,"//eexpmap//",junknorm",nnormmap,"((im1*im2)**2.)/im3",nullval=0.,ver-)
	minmax(nnormmap,force+,update+,ver-)
	imar(nnormmap,"/",minmax.maxval,nnormmap,ver-,noact-)
hedit(nnormmap,"title","Normalization map for "//ooutimage,add+,ve-,sho-,updat+)
	imdel("junkweight."//oextimage,ver-)
	imdel("junknorm."//oextimage,ver-)
	imdel(nnormmap//"num.fits",ver-)
	imdel(nnormmap//"pl.pl",ver-)

if(flip){
  imcopy(nnormmap//"[-*,*]", nnormmap, ver-)
  addcomment(nnormmap,"Image has been flipped L-R from orig. orientation",ver-)
}

} else {
	print("Output normalization map already exists-- new one not created.")
 	print("Output normalization map already exists-- new one not created.", >> llogfile)
}


# UNDO PRE-ZEROING: if zeroing was done before imcombine, undo it if desired
if(pprezero && uunzero){
# If block-replication was done, skip step if those images are to be deleted
	if(mag!=1 && ccleanup && !noappend){
	    print("", >> llogfile)
	} else {
	    print("===> Undoing zero offsetting...")
	    list=list_of_zeros
	    while (fscan (list,img,levelzero) != EOF) {
	        imgets(img,"PREZERO",>&"dev$null")
	        if(imgets.value!="0"){
	            	tmpzero=abs((real(imgets.value)-levelzero)/levelzero)
			if(tmpzero>=0.01){
print("WARNING! "//img//" pre-zeroing doesn't match list to 1%; unzeroing not done")
			} else {
			  levelzero=real(imgets.value)
		        imar(img,"-",levelzero,img,pixt="real",calc="real",ver-)
		     hedit(img,"PREZERO","",del+,ver-,show-,updat+,>&"dev$null")
			  }
                } else {
	      print("No action -- "//img//" not pre-zeroed in the first place!")
	        }
	    }
	}
}
if(pprezero){
       if(sstack && access(sstackimage//"."//oextimage)){print("", >> llogfile)}
	else{del(list_of_zeros,ver-,>&"dev$null")}
}
list=""


# UNDO PRE-SCALING: if scaling was done before imcombine, undo it if desired
if(pprescale && uunscale){
# If block-replication was done, skip step if those images are to be deleted
	if(mag!=1 && ccleanup && !noappend){
		print("", >> llogfile)
	} else {
		print("===> Undoing scaling factors...")
		list=list_of_scales
		while (fscan (list,img,sf) != EOF) {
			imgets(img,"PREZERO",>&"dev$null")
			if(imgets.value!="0"){
      print("WARNING! "//img//" still pre-scaled: pre-zeroing not undone first")
			} else {
		    	  imgets(img,"PRESCALE",>&"dev$null")
		    	  if(imgets.value!="0"){
			    tmpscale=abs((real(imgets.value)-sf)/sf)
			    if(tmpscale>=0.01){
print("WARNING! "//img//" pre-scaling doesn't match list to 1%; unscaling not done")
			    } else {
			      imar(img,"/",sf,img,pixt="real",calct="real",ver-)
		     hedit(img,"PRESCALE","",del+,ver-,sho-,updat+,>&"dev$null")
			      if(ddefscale){
				imgets(img,"AIRMOLD",>&"dev$null")
       hedit(img,"AIRMASS",imgets.value,add+,del-,ver-,sho-,updat+,>&"dev$null")
		      hedit(img,"AIRMOLD","",del+,ver-,sho-,updat+,>&"dev$null")
			      }
			    }
			  } else {
	      print("No action -- "//img//" not pre-scaled in the first place!")
			  }
			}
		}
	}
}
if(pprescale)del(list_of_scales,ver-,>&"dev$null")
list=""


# IMSHIFTS FILE
# Create *.imshifts file, listing images & offsets used by masklet & photcheck
if (!access(ooutimage//".imshifts")) {
	print("===> Creating output imshifts file...")
	if (!access(ccomblist2))copy(ccomblist1,ccomblist2,ver-)
	join(ccomblist2,sshiftintlist,output=ooutimage//".imshifts_tmp")
# Find minimum x and y shift values and subtract them from output offsets
        listimshift=ooutimage//".imshifts_tmp"
	list=""
	list=listimshift
	xmin=100000
	ymin=100000
        while (fscan (list,img,newx,newy) != EOF) {
               	if (newx < xmin)  {
                       	xmin = newx
               	}
               	if (newy < ymin)  {
                       	ymin = newy
               	}
	}
	newx=0
	newy=0
	list=""
	list=listimshift
        while (fscan (list,img,oldx,oldy) != EOF) {
		newx = oldx - xmin
		newy = oldy - ymin
		print(img, " ", newx, newy, >> ooutimage//".imshifts")	
	}
	del(ooutimage//".imshifts_tmp",ver-)
} else {
	print("Output imshifts file already exists-- not overwritten.")
}

# Clean up.
if(mag!=1){
	if(ccleanup && !noappend){
		if(access(bprotlist)){
			imdel("@"//bprotlist,ver-,>&"dev$null")
			del(bprotlist,ver-,>&"dev$null")
		}
		if(access(ccomblist2))imdel("@"//ccomblist2,ver-,>&"dev$null")
		del(ccomblist2,ver-,>&"dev$null")
		del(sshiftexactlist//blk,ver-,>&"dev$null")
		del(zzeroname//blk,ver-,>&"dev$null")
	}
} else {
	del(ccomblist2,ver-,>&"dev$null")
}

print(" ")
print(ooutimage//", "//eexpmap//" (exposure map), imshifts file,")
print("and "//nnormmap//" (normalization map) have been created.")
#print("Next task is `mkobjmask' or `photcheck'.")

} else {
        print("WARNING: tasks stsdas.toolbox.imgtools.imcalc, addcomment,")
	print("filecalc, fileroot, and/or iterstat are not loaded!")
	print("Please load the appropriate package(s) and restart this task.")
}

list=""
print(" ")
if(echog)beep

end

