
#in all of this, 1234 are done in parallel but separately, individually.
#x,y coords are first generated using irshift (BRI separate)
#coadding shifts the x,y (BRI separate)
#maskaligntrim shifts them again (BRI should end up on approx. same system)

#irshift
#btc_coadd.cl
#btc_maskaligntrim.cl

procedure btc_coadd(inlist, outimg, outplf, offfile, logname)

# btc_coadd -- coadd BTC images using standard parameters
# 980805: created

string  inlist  {prompt="List of images to coadd"}
string  outimg	{prompt="Output coadded image name"}
string	outplf	{prompt="Output pixel list file name"}
string	offfile	{prompt="File with image offsets for coadding"}
string	logname	{"STDOUT",prompt="Log file"}
string	imsect  {"[950:1100,950:1100]",prompt="Section of image to use for makeobjmask statistics"}
bool	delold	{no,prompt="Delete preexisting copies of intermediate files?"}
bool	cleanup	{yes,prompt="Delete intermediate images when done?"}
bool	echog	{yes,prompt="Beep when task is done?"}
 
begin

string	in_list, outimage, off_file, log_name, plfimage, im_sect, extn
int	nlines1, nlines2
bool	clean_up, del_old

in_list = inlist
fileroot(outimg,validim+)
outimage = fileroot.root
extn = fileroot.extension
if(extn=="")extn="fits"
fileroot(outplf,validim+)
plfimage = fileroot.root
off_file = offfile
log_name = logname
	if (log_name=="") log_name="STDOUT"
im_sect = imsect
clean_up = cleanup
del_old = delold

count (in_list) | scan (nlines1)
count (off_file) | scan (nlines2)

if (nlines1!=nlines2) {
	beep
	print ("Task aborted -- number of images & shifts don't match!")
} else {
# make plf file
    print("", >> log_name)
    if (delold) {
	imdel (plfimage, ver-, >& "dev$null")
    }
    if (!access(plfimage//".pl")) {
	print("===> Making PLF image...", >> log_name)
	imcombine ("@"//in_list, outimage//"avgtmp", plf=plfimage, logf=log_name, offsets=off_file, rej="none", scale="none", zero="mode", weight="exposure", maskt="none")
	imdel (outimage//"avgtmp", ver-)
    }
# make average coadd
    if (delold) {
	imdel (outimage//"avg", ver-, >& "dev$null")
    }
    if (!access(outimage//"avg."//extn)) {
	print("===> Making average coadd...", >> log_name)
	imcombine ("@"//in_list, outimage//"avg", plf="", logf=log_name, offsets=off_file, rej="none", scale="none", zero="mode", weight="exposure")
    }
# make spurious-object-cleaned coadd
    if (delold) {
	imdel (outimage//"rej", ver-, >& "dev$null")
    }
    if (!access(outimage//"rej."//extn)) {
	print("===> Making average coadd...", >> log_name)
	imcombine ("@"//in_list, outimage//"rej", plf="", logf=log_name, offsets=off_file, rej="ccdclip", hsig=4, lsig=10, grow=2, scale="none", zero="mode", weight="exposure", nkeep=1)
    }
# make spurious-object-cleaned coadd object mask (objects=1)
    if (delold) {
	imdel (outimage//"rejominv.pl", ver-, >& "dev$null")
    }
    if (!access(outimage//"rejominv.pl")) {
	makeobjmask (outimage//"rej", subsam=3, filtsize=10, suffix="om", nsmoo=3, statsec=im_sect, ngrow=2, inv-)
	imren (outimage//"rejom.pl", outimage//"rejominv.pl")
    }
# create image of objects only
    if (delold) {
	imdel (outimage, ver-, >& "dev$null")
    }
    if (!access(outimage//"."//extn)) {
	imar (outimage//"avg", "*", outimage//"rejominv.pl", outimage, noact-)
    }
# invert spurious-object-cleaned coadd object mask so objects=0
    if (delold) {
	imdel (outimage, ver-, >& "dev$null")
    }
    if (!access(outimage//"rejom.pl")) {
	minv (outimage//"rejominv.pl", outimage//"rejom.pl")
    }
# create image of sky only
    if (delold) {
	imdel (outimage//"rejobjn."//extn, ver-, >& "dev$null")
    }
    if (!access(outimage//"rejobjn."//extn)) {
	imar (outimage//"rej", "*", outimage//"rejom.pl", outimage//"rejobjn", noact-)
    }
# add sky-only and object-only images
	imar (outimage, "+", outimage//"rejobjn", outimage, noact-)
# add exposure image
	hedit (outimage, "PLFMASK", plfimage, add+, ver-, show+, update+)
# clean up
	imdel (outimage//"rejobjn", ver-)
	if (clean_up) {
		imdel (outimage//"rej", ver-)
		imdel (outimage//"avg", ver-)
	}
	if(echog)beep
}

end


